/* BUGREPIF.H: FMH BugRep interface routines
*/

#ifndef __BUGREPIF_H
#define __BUGREPIF_H

/* This is a sample implementation of interface routines
   to FMH BugRep. To keep it simple, a strict order for calling
   the routines must be followed:

   brNew
   brMsgBoxText (repeated, optional)
   brMsgBoxList (repeated, optional)
   brMailText (repeated, optional)
   brMailFile or brMailAttach/brMailAttachData (repeated, optional)
   brSubmit
*/

#ifdef __cplusplus
extern "C" {
#endif

int brActive(void);
/* checks wether FMH BugRep is running and active or not
   returns !=0 if active, ==0 if not
*/

void brNew(void);
/* starts a new bug report */

void brMsgBoxText(char *text, ...);
/* adds text to the error message box (normal text field) */

void brMsgBoxList(char *text, ...);
/* like before, but for the list box; submits one line per call */

void brMailText(char *msg, ...);
/* submits text for email message (ISO charset) */

void brMailFile(char *attFn, char *realFn);
/* attaches an external file */

void brMailAttach(char *attFn);
/* starts attaching an internal file (program generated) */

void brMailAttachData(void *data, int size);
/* adds data to the current internal file */

void brSubmit(void);
/* submits current bug report */

#ifdef __cplusplus
};
#endif

#endif /* _BUGREPIF_H */
