// XASSERT.C: extended assert macro

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "xassert.h"
#include "bugrepif.h"

XASSERT_ID

#ifndef NDEBUG

static void brMailAttachText(char *s)
{
  brMailAttachData(s,strlen(s));
}

static void dumpAssertIDs(void)
// searches current process for all Assert IDs and dumps them into the
// current mail file as simple attachment
{
  MEMORY_BASIC_INFORMATION mbi;
  DWORD hMod;
  PIMAGE_DOS_HEADER pDosHdr;
  PIMAGE_NT_HEADERS pNtHdr;
  PIMAGE_SECTION_HEADER pSection;
  unsigned i;

  // start attachment
  brMailAttach("FILELIST.TXT");
  brMailAttachText("List of Assert IDs in program\r\n"
                   "-----------------------------\r\n");

  if (!VirtualQuery((PVOID)&dumpAssertIDs,&mbi,sizeof(mbi)))
    return;
  hMod=(DWORD)mbi.AllocationBase;
  pDosHdr=(PIMAGE_DOS_HEADER)hMod;
  pNtHdr=(PIMAGE_NT_HEADERS)(hMod+pDosHdr->e_lfanew);
  pSection=IMAGE_FIRST_SECTION(pNtHdr);
  for (i=0;i<pNtHdr->FileHeader.NumberOfSections;i++,pSection++)
  {
    char *cur;
    unsigned rest;

    if (!(pSection->Characteristics&IMAGE_SCN_CNT_INITIALIZED_DATA))
      continue;

    // search for IDs
    cur=(char *)(hMod+pSection->VirtualAddress);
    rest=max(pSection->SizeOfRawData,pSection->Misc.VirtualSize);
    while (rest>sizeof(tXAssertID))
    {
      tXAssertID *id;
      void *res=memchr(cur,'$',rest-sizeof(tXAssertID)+1);

      if (!res) break;
      rest-=((DWORD)res)-((DWORD)cur);
      cur=(char *)res;
      id=(tXAssertID *)res;
      if (!memcmp(id->id,"$@XASS@$",8))
        if (id->self==id)
        {
          char buf[200];
          sprintf(buf,"%-60s %s %s\r\n",id->file,id->date,id->time);
          brMailAttachText(buf);
        }
      cur++;
      rest--;
    }
  }
}

void __x_assert(char* date, char* time, int cond, char* expr, char* file,
                int line, tXAssertFunc func, void* par, char* text, ...)
{
  static char extra[500];

  if (cond) return;

  // evaluate additional text format string
  if (text)
  {
    va_list va;

    va_start(va,text);
    vsprintf(extra,text,va);
    va_end(va);
  }
  else *extra=0;

  // condition failed, generate bug report and end program
  if (brActive())
  {
    brNew();
    brMsgBoxText("An assertion inside the current process has failed. Since "
                 "this might indicate a serious program bug the process has been "
                 "terminated. You may send a bug report via email "
                 "if you want to.");
    brMsgBoxList("Expression: %s",expr);
    brMsgBoxList("File: %s, line %i (%s %s)",file,line,date,time);
    if (text) 
    {
      char *s=extra;
      while (*s)
      {
        char *q=s,more;
        while (*q&&*q!='\n') q++;
        more=*q; *q=0;
        brMsgBoxList(s);
        *q=more;
        s=more?q+1:q;
      }
    }

    brMailText("Assertion failed\n"
               "Expr: %s\n"
               "File: %s, line %i (%s %s)\n",expr,file,line,date,time);
    if (text)
      brMailText(extra);
    if (func)
      func(par);
    dumpAssertIDs();
    brSubmit();
  }
  else
  {
    // BugRep inactive, generate standard message box
    static char msg[1000];

    sprintf(msg,"Condition: %s\n"
                "File: %s, line %i"
                "%s%s",expr,file,line,text?"\n":"",extra);
    MessageBox(NULL,msg,"ASSERTION FAILED",
                MB_OK|MB_ICONERROR|MB_TASKMODAL|MB_SETFOREGROUND);
  }

  // that's it folks!
  #ifdef __WATCOMC__
    abort();
  #else
    _exit(3);
  #endif
}

#endif /* NDEBUG */
